/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief
 *      Functions for use of the Universal Serial Interface
 *
 *      Contains high level functions for initializing the USI as an SPI slave,
 *      interrupt handling, sending and receiving single bytes.
 *
 * \par Application note:
 *      AVR458: Charging Li-Ion Batteries with BC100 \n
 *      AVR463: Charging NiMH Batteries with BC100
 *
 * \par Documentation:
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com \n
 *      Original author: \n
 *
 * $Name$
 * $Revision: 2299 $
 * $RCSfile$
 * $URL: http://svn.norway.atmel.com/AppsAVR8/avr458_Charging_Li-Ion_Batteries_with_BC100/tag/20070904_release_1.0/code/IAR/USI.c $
 * $Date: 2007-08-23 12:55:51 +0200 (to, 23 aug 2007) $\n
 ******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>

#include "enums.h"
#include "structs.h"

#include "main.h"
#include "ADC.h"
#include "battery.h"
#include "time.h"
#include "USI.h"


//******************************************************************************
// Variables
//******************************************************************************
//! SPI status struct
volatile SPI_Status_t SPI;


//******************************************************************************
//Functions
//*****************************************************************************
/*! \brief USI Counter Overflow Interrupt Service Routine
 *
 * When the USI counter overflows, a byte has been transferred.\n
 * The USIDR contents are stored and flags are updated.
 *
 * The protocol is quite simple and has three sequential states: command,
 * address and data.
 * (Keep in mind that the Master is in charge of data clocking, which means
 * there is a one byte "delay" from when the Slave puts something to SPI till
 * the Master can read it.)
 *
 * 1. If a non-zero byte is received in the command state, the ISR will
 * store the commands to the SPI struct (read/write, EEPROM/SRAM, number of
 * bytes). To signal that the command was received, 0xCC is put to the SPI bus.
 * If a zero byte (0x00) is received in the command state, it is simply ignored
 * because it is an invalid command.
 *
 * 2. When a byte is received in the address state, it is stored to the SPI
 * struct. To signal that the address was received, 0xBB is put to SPI bus.
 * 
 * 3. In the data state, variables are read/written "from back to front" until
 * the byte counter reaches zero. Since the Master is in charge of the data
 * clocking, the Slave will go to command state before the last byte is
 * transferred during reading. This means that the Master should send an 
 * invalid command when getting each byte, ie 0x00.
 *
 * If the time between two transfers is 1 second or more, the Slave
 * automatically reverts to command state.
 *
 * \note Battery charging is not automatically halted during SPI communication.
 * This means that the current charge state (current and voltage) will
 * remain constant during heavy and prolonged serial traffic.
 *
 * \todo Variable writing not implemented yet.
 * \todo EEPROM/SRAM flag doesn't really do anything with this implementation.
 */
ISR(USI_OVF_vect)
{
	// If the communication timed out, set ST_CMD as current state.
	if (!Time_Left(TIMER_USI)) {
		SPI.State = ST_CMD;
	}

	// Start communication timer. If further communication doesn't happen
	// within 1 second, the SPI communication state is reset to CMD.
	Time_Set(TIMER_USI, 0, 1, 0);
	
	// Clear USI counter and flag completed transfer.
	USISR = (1<<USIOIF);
	SPI.XferComplete = TRUE;
	
	// Process incoming data.
	switch(SPI.State)	{
	// A valid SPI transfer starts with a Command Byte sent by the Master.
	case ST_CMD:   
		SPI.Data = USIDR;  // Store the transferred byte.
		
		// If the master sent 0, it is trying to get data. Ignore in this state.
		if (SPI.Data != 0) {
			// Does the master want to read or write?
			if (SPI.Data & 0x40) {
				SPI.Read = FALSE;
			} else {
				SPI.Read = TRUE;
			}

				// From/to EEPROM or SRAM?
			if (SPI.Data &0x80) {
				SPI.EEPROM = TRUE;
			} else {
				SPI.EEPROM = FALSE;
			}

			SPI.Count = (SPI.Data & 0x3F);  // Get number of bytes to receive/send.
			SPI.State = ST_ADDR;  // The Master will send the address byte next.

			SPI_Put(0xCC);  // Signal that command was received.
		}
	break;

			
	case ST_ADDR:  
		SPI.Data = USIDR;  // Store the address.
		SPI.Address = SPI.Data;
		SPI.State = ST_DATA;  // The master will send/wait for data next.

		SPI_Put(0xBB);  // Signal that address was received.
	break;


	// Note well: this will process at least one byte, regardless of Count.
	case ST_DATA:
		if (SPI.Count-- > 0) {
			// Write specified variable to SPI, "back to front".
			if (SPI.Read) {
				switch (SPI.Address) {
				case ADR_ADCS:
					SPI_Put(*(((unsigned char*)&ADCS) + (SPI.Count)));
				break;
				
				
				case ADR_BATTACTIVE:
					SPI_Put(*((unsigned char*)&BattActive + (SPI.Count)));
				break;
				
				
				case ADR_BATTDATA:
					SPI_Put(*((unsigned char*)&BattData + (SPI.Count)));
				break;
				
				
				case ADR_BATTCTRL:
					SPI_Put(eeprom_read_byte((unsigned char*)&BattControl + (SPI.Count)));
				break;
				
				case ADR_TIMERS:
					SPI_Put(*((unsigned char*)&timeval + (SPI.Count)));
				break;

				
				default:
					SPI_Put(0);
				break;
				}
			} else {
				// Read byte from SPI
				SPI.Data = USIDR;

				// ********************************************
				// THIS FUNCTION HAS NOT BEEN FULLY IMPLEMENTED
				// ********************************************
								
				// Save byte to specified variable.
				switch (SPI.Address) {
				case ADR_BATTCTRL:
					eeprom_write_byte((unsigned char*)&BattControl + SPI.Count, SPI.Data);
				break;

				
				default:
				break;
				}
			}
			

		} else {
			SPI.State = ST_CMD;
		}
	break;

	default:  // Shouldn't end up here. (Unknown SPI-state)
	break;
	}
}


/*! \brief Initializes USI as an SPI slave
 *
 * Initializes USI as a 3-wire SPI slave using the pins specified in USI.h for
 * I/O and clock, and USI counter overflow interrupts enabled.\n
 * Also initializes the SPI status struct.
 * 
 * \param spi_mode Specifies if USI should trigger on positive (0) or negative
 * (1) edge of clock signal
 *
 * \note Clears the stored data
 *
 * \todo Timer should reset SPI protocol on timeout
 */
void SPI_Init(unsigned char spi_mode)
{
	unsigned char sreg_saved;

	sreg_saved = SREG;
	cli();
	
	// Configure outputs and inputs, enable pull-ups for DATAIN and CLOCK pins.
	USI_DIR_REG |= (1<<USI_DATAOUT_PIN);
	USI_DIR_REG &= ~((1<<USI_DATAIN_PIN) | (1<<USI_CLOCK_PIN));
	USI_OUT_REG |= (1<<USI_DATAIN_PIN) | (1<<USI_CLOCK_PIN);
	
	// Configure USI to 3-wire slave mode with overflow interrupt
	USICR = ( (1<<USIOIE) | (1<<USIWM0) | (1<<USICS1) | (spi_mode<<USICS0) );

	// Initialize the SPI struct
	SPI.Data = 0;                 // Clear data.
	SPI.State = ST_CMD;           // Initial SPI state: wait for command.
	SPI.Read = FALSE;             // Doesn't matter right now.
	SPI.EEPROM = FALSE;           // Doesn't matter right now.
	SPI.Count = 0;                // Doesn't matter right now.
	SPI.Address = 0;              // Doesn't matter right now.
	SPI.XferComplete = FALSE;     // We haven't even started a transfer yet.
	SPI.WriteCollision = FALSE;   // ..And therefore a collision hasn't happened.

	SREG = sreg_saved;
}


// Put one byte on bus. Use this function like you would write to the SPDR
// register in the native SPI module. Calling this function will prepare a
// byte for the next transfer initiated by the master device. If a transfer
// is in progress, this function will set the write collision flag and return
// without altering the data registers.
//
// Returns 0 if a write collision occurred, 1 otherwise.
/*! \brief Write a byte to SPI bus
 *
 * This function first checks if a transmission is in progress, and if so, flags
 * a write collision, and returns FALSE.\n
 * If a transmission is not in progress, the flags for write collision and
 * transfer complete are cleared, and the input byte is written to SPDR.\n
 *
 * \param val The byte to send.
 *
 * \retval FALSE A write collision happened.
 * \retval TRUE Byte written to SPDR.
 */
unsigned char SPI_Put(unsigned char val)
{
	// Check if transmission in progress, i.e. if USI counter doesn't equal zero.
	// If this fails, flag a write collision and return.
	if((USISR & 0x0F) != 0) {
		SPI.WriteCollision = TRUE;
		return(FALSE);
	}

	// Reinitialize flags.
	SPI.XferComplete = FALSE;
	SPI.WriteCollision = FALSE;

	USIDR = val;  // Put data in USI data register.

	return (TRUE);
}


// Get one byte from bus. This function only returns the previous stored
// USIDR value. The transfer complete flag is not checked. Use this function
// like you would read from the SPDR register in the native SPI module.
/*! \brief Get the last byte received from SPI bus
 *
 * This function simply returns the last byte stored to the SPI status struct,
 * without checking if a completed transfer is flagged.
 *
 * \retval SPI.Data The last byte read from SPI.
 */
unsigned char SPI_Get(void)
{
	return SPI.Data;
}


/*! \brief Wait for SPI transfer to complete
 *
 * This function waits for a transfer complete to be flagged.
 */
void SPI_Wait(void)
{
	do {  // Wait for transfer complete.
	} while (SPI.XferComplete == FALSE);
}
