/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief
 *      Main program file
 *
 *      Contains the main program, which is a basic state machine.
 *
 * \par Application note:
 *      AVR458: Charging Li-Ion Batteries with BC100 \n
 *      AVR463: Charging NiMH Batteries with BC100
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler 
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * 
 * $Name$
 * $Revision: 2302 $
 * $RCSfile$
 * $URL: http://svn.norway.atmel.com/AppsAVR8/avr458_Charging_Li-Ion_Batteries_with_BC100/tag/20070904_release_1.0/code/IAR/main.c $
 * $Date: 2007-08-23 14:57:36 +0200 (to, 23 aug 2007) $\n
 ******************************************************************************/

/*! \page License
 * Copyright (c) 2007, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <avr/io.h>
#include <avr/pgmspace.h>

#include <stdlib.h>

#include "structs.h"

#include "main.h"
#include "ADC.h"
#include "statefunc.h"
#include "battery.h"
#include "menu.h"
#include "OWI.h"
#include "PWM.h"
#include "time.h"
#include "USI.h"



//******************************************************************************
// Globals
//******************************************************************************
unsigned char CurrentState;     //!< \brief Global that indicates current state
                                //!<
                                //!< Updated by main().
                                //!< \note See menu.h for definition of states.


//******************************************************************************
// Functions
//******************************************************************************
/*! \brief Main program
 *
 * The main function goes into an infinite loop, keeping track of the current
 * state and the next one. If the next state is different from the current, it
 * looks up the address to the next state function, in \ref menu_state[], and 
 * updates \ref CurrentState. The state function is then called and will
 * eventually return a new state, and so the loop reiterates.
 *
 * \todo The variable inp is passed to all state functions, but is not used
 * for anything yet. Remove?
 */
int main( void )
{
	unsigned char nextstate, inp, i;
	unsigned char (*pStateFunc)(unsigned char);   // Function pointer.
	
	// Initialize local state variables.
	inp = ZERO;
	CurrentState = nextstate = ST_INIT;
	pStateFunc = NULL;
	
	// Look for function associated with current state, get its address.
	for (i = 0; pgm_read_byte(&menu_state[i].state) != 0; i++) {
		if (pgm_read_byte(&menu_state[i].state) == CurrentState) {
			pStateFunc = (PGM_VOID_P) pgm_read_word(&menu_state[i].pFunc);
		}
	}

	while (TRUE) {
		// Run function associated with current state, get next state in return.
		if (pStateFunc != NULL){
			nextstate = pStateFunc(inp);
		}

		// Look up function for next state, if it differs from the current.
		if (nextstate != CurrentState) {
			CurrentState = nextstate;
			for ( i = 0; pgm_read_byte(&menu_state[i].state) != 0; i++) {
				if (pgm_read_byte(&menu_state[i].state) == CurrentState) {
					pStateFunc = (PGM_VOID_P) pgm_read_word(&menu_state[i].pFunc);
				}
			}
		}
	}
}


/* Doxygen documentation mainpage ********************************************/
/*! \mainpage
 * \section intro Introduction
 * This documents the software for application note AVR458. This is a charger
 * for Li-Ion batteries, based on ATAVRBC100.
 *
 * \section compinfo Compilation Info
 * This software was compiled with IAR Embedded Workbench, 4.30. To use GCC 
 * the source have to be modified.\n
 * \n
 * To make project on IAR EWAVR:\n
 * Add the .c files to project (ADC.c, battery.c, main.c, menu.c,
 * OWI.c, PWM.c, time.c and USI.c). Add either LIIONcharge.c or NIMHcharge.c,
 * and update LIIONspecs.h or NIMHspecs.h, and battery.h with the appropriate
 * battery data if needed.\n
 * \n
 * Use device --cpu=tiny861, enable bit definitions in I/O include files,
 * optimization low for debug target and high for release, output format: ubrof8
 * for Debug and intel_extended for Release. \n
 *
 * \section deviceinfo Device Info
 * This application is based on the ATtiny 861, but it is possible to migrate 
 * the design to other AVR microcontrollers, such as pin-compatible devices
 * ATtiny 261/461. Low pin count devices such as ATtiny 25/45/85 may also be
 * used, but with reduced functionality.
 * 
 * Required fuse bit settings:
 *    <pre>
 *    FUSE BIT  | SETTING
 *    ----------+--------------------
 *    CKDIV8    | 1 (unprogrammed)
 *    CKSEL3..0 | 0010 (internal osc)
 *    </pre>
 *
 *
 * \section todo To Do-list
 * \todo
 * - Finalize master-slave communication protocol
 * - Implement discharge mode
 *
 * \section contactinfo Contact Info
 * For more info about Atmel AVR visit http://www.atmel.com/products/AVR/ \n
 * For application notes visit 
 * http://www.atmel.com/dyn/products/app_notes.asp?family_id=607 \n
 * Support mail: avr@atmel.com
 */


/*! \page misra MISRA C rule violations
 *
 * \par Rule 1
 * <i>"All code shall conform to ISO 9899 standard C, with no extensions
 * permitted."</i>
 *
 * Extensions are necessary because ISO C has no way of specifying that a
 * function should be an interrupt service routine, or that we would like data
 * members to be stored in f.ex. EEPROM.
 *
 *
 * \par Rule 37
 * <i>"Bitwise operations shall not be performed on signed integer types."</i>
 *
 * The compiler assumes all the 1's we shift around to make bitmasks are signed
 * integers. Specifying them all to be (unsigned int), either directly or via a
 * definition, would fix the rule violations, but also reduce code readability.
 *
 *
 * \par Rule 45
 * <i>"Type casting from any type to or from pointers shall not be used."</i>
 *
 * Assigning macro NULL, defined in stdlib.h, to a pointer causes this..
 *
 *
 * \par Rule 96
 * <i>"In the definition of a function-like macro the whole definition, and each
 * instance of a parameter, shall be enclosed in parentheses."</i>
 * 
 * It is difficult to use parentheses with void function-like macros.
 */
