/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file ********************************************************************
 *
 * \brief
 *      Functions for 1-Wire(R) bus communication
 *
 *      High level functions for transmission of full bytes on the 1-Wire(R)
 *      bus and implementations of ROM commands.\n
 *      Polled software only implementation of the basic bit-level signalling
 *      in the 1-Wire(R) protocol.\n
 *      Includes functions for computing and checking CRC8 & 16 values of data
 *      sets, and of 64 bit ROM identifiers.
 *      Supported devices:  All AVRs.
 *
 * \par Application Note:
 *      AVR458: Charging Li-Ion Batteries with BC100\n
 *      AVR463: Charging NiMH Batteries with BC100\n
 *      One-wire protocol based on AVR318 - Dallas 1-Wire(R) master.
 *
 * \par Documentation:
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com \n
 *      Original author: \n
 *
 * $Name$
 * $Revision: 2299 $
 * $RCSfile$
 * $URL: http://svn.norway.atmel.com/AppsAVR8/avr458_Charging_Li-Ion_Batteries_with_BC100/tag/20070904_release_1.0/code/IAR/OWI.c $
 * $Date: 2007-08-23 12:55:51 +0200 (to, 23 aug 2007) $\n
****************************************************************************/

#include <ioavr.h>
#include <inavr.h>

#include "OWI.h"


//******************************************************************************
// Functions
//******************************************************************************
/*! \brief Initialization of the one wire bus(es). (Software only driver)
 *  
 *  This function initializes the 1-Wire bus(es) by releasing it and
 *  waiting until any presence signals are finished.
 *
 *  \param  pins    A bitmask of the buses to initialize.
 */
void OWI_Init(unsigned char pins){
	OWI_RELEASE_BUS(pins);
	// The first rising edge can be interpreted by a slave as the end of a
	// Reset-pulse. Delay for the required reset recovery time (H) to be 
	// sure that the real reset is interpreted correctly.
	__delay_cycles(OWI_DELAY_H_STD_MODE);
}


/*! \brief  Write a '1' bit to the bus(es). (Software only driver)
 *
 *  Generates the waveform for transmission of a '1' bit on the 1-Wire
 *  bus.
 *
 *  \param  pins    A bitmask of the buses to write to.
 */
void OWI_WriteBit1(unsigned char pins){
	unsigned char intState;
	
	// Disable interrupts.
	intState = __save_interrupt();
	__disable_interrupt();
	
	// Drive bus low and delay.
	OWI_PULL_BUS_LOW(pins);
	__delay_cycles(OWI_DELAY_A_STD_MODE);
	
	// Release bus and delay.
	OWI_RELEASE_BUS(pins);
	__delay_cycles(OWI_DELAY_B_STD_MODE);
	
	// Restore interrupts.
	__restore_interrupt(intState);
}

/*! \brief  Write a '0' to the bus(es). (Software only driver)
 *
 *  Generates the waveform for transmission of a '0' bit on the 1-Wire(R)
 *  bus.
 *
 *  \param  pins    A bitmask of the buses to write to.
 */
void OWI_WriteBit0(unsigned char pins)
{
	unsigned char intState;
	
	// Disable interrupts.
	intState = __save_interrupt();
	__disable_interrupt();
	
	// Drive bus low and delay.
	OWI_PULL_BUS_LOW(pins);
	__delay_cycles(OWI_DELAY_C_STD_MODE);
	
	// Release bus and delay.
	OWI_RELEASE_BUS(pins);
	__delay_cycles(OWI_DELAY_D_STD_MODE);
	
	// Restore interrupts.
	__restore_interrupt(intState);
}

/*! \brief  Read a bit from the bus(es). (Software only driver)
 *
 *  Generates the waveform for reception of a bit on the 1-Wire(R) bus(es).
 *
 *  \param  pins    A bitmask of the bus(es) to read from.
 *
 *  \return A bitmask of the buses where a '1' was read.
 */
unsigned char OWI_ReadBit(unsigned char pins)
{
	unsigned char intState;
	unsigned char bitsRead;
	
	// Disable interrupts.
	intState = __save_interrupt();
	__disable_interrupt();
	
	// Drive bus low and delay.
	OWI_PULL_BUS_LOW(pins);
	__delay_cycles(OWI_DELAY_A_STD_MODE);
	
	// Release bus and delay.
	OWI_RELEASE_BUS(pins);
	__delay_cycles(OWI_DELAY_E_STD_MODE);
	
	// Sample bus and delay.
	bitsRead = OWI_PIN & pins;
	__delay_cycles(OWI_DELAY_F_STD_MODE);
	
	// Restore interrupts.
	__restore_interrupt(intState);
	
	return bitsRead;
}


/*! \brief  Send a Reset signal and listen for Presence signal. (software
 *  only driver)
 *
 *  Generates the waveform for transmission of a Reset pulse on the 
 *  1-Wire(R) bus and listens for presence signals.
 *
 *  \param  pins    A bitmask of the buses to send the Reset signal on.
 *
 *  \return A bitmask of the buses where a presence signal was detected.
 */
unsigned char OWI_DetectPresence(unsigned char pins)
{
	unsigned char intState;
	unsigned char presenceDetected;
	
	// Disable interrupts.
	intState = __save_interrupt();
	__disable_interrupt();
	
	// Drive bus low and delay.
	OWI_PULL_BUS_LOW(pins);
	__delay_cycles(OWI_DELAY_H_STD_MODE);
	
	// Release bus and delay.
	OWI_RELEASE_BUS(pins);
	__delay_cycles(OWI_DELAY_I_STD_MODE);
	
	// Sample bus to detect presence signal and delay.
	presenceDetected = ((~OWI_PIN) & pins);
	__delay_cycles(OWI_DELAY_J_STD_MODE);
	
	// Restore interrupts.
	__restore_interrupt(intState);
	
	return presenceDetected;
}


/*! \brief  Sends one byte of data on the 1-Wire(R) bus(es).
 *  
 *  This function automates the task of sending a complete byte
 *  of data on the 1-Wire bus(es).
 *
 *  \param  data    The data to send on the bus(es).
 *  
 *  \param  pins    A bitmask of the buses to send the data to.
 */
void OWI_SendByte(unsigned char data, unsigned char pins)
{
	unsigned char temp;
	unsigned char i;
	
	// Do once for each bit
	for (i = 0; i < 8; i++) {
		// Determine if LSB is '0' or '1' and transmit corresponding
		// waveform on the bus.
		temp = data & 0x01;
		
		if (temp) {
			OWI_WriteBit1(pins);
		} else {
			OWI_WriteBit0(pins);
		}
	
		data >>= 1;  // Right shift the data to get next bit.
	}
}


/*! \brief  Receives one byte of data from the 1-Wire(R) bus.
 *
 *  This function automates the task of receiving a complete byte 
 *  of data from the 1-Wire bus.
 *
 *  \param  pin     A bitmask of the bus to read from.
 *  
 *  \return     The byte read from the bus.
 */
unsigned char OWI_ReceiveByte(unsigned char pin)
{
	unsigned char data;
	unsigned char i;
	
	// Clear the temporary input variable.
	data = 0x00;
	
	// Do once for each bit
	for (i = 0; i < 8; i++) {
		// Shift temporary input variable right.
		data >>= 1;
		
		// Set the MSB if a '1' value is read from the bus.
		// Leave as it is ('0') else.
		if (OWI_ReadBit(pin)) {
			data |= 0x80;
		}
	}
	
	return data;
}


/*! \brief  Sends the SKIP ROM command to the 1-Wire bus(es).
 *
 *  \param  pins    A bitmask of the buses to send the SKIP ROM command to.
 */
void OWI_SkipRom(unsigned char pins)
{
	// Send the SKIP ROM command on the bus.
	OWI_SendByte(OWI_ROM_SKIP, pins);
}


/*! \brief  Sends the READ ROM command and reads back the ROM id.
 *
 *  \param  romValue    A pointer where the id will be placed.
 *
 *  \param  pin     A bitmask of the bus to read from.
 */
void OWI_ReadRom(unsigned char * romValue, unsigned char pin)
{
	unsigned char bytesLeft = 8;
	
	// Send the READ ROM command on the bus.
	OWI_SendByte(OWI_ROM_READ, pin);
	
	// Do 8 times.
	while (bytesLeft > 0) {
		// Place the received data in memory.
		*romValue++ = OWI_ReceiveByte(pin);
		bytesLeft--;
	}
}


/*! \brief  Sends the MATCH ROM command and the ROM id to match against.
 *
 *  \param  romValue    A pointer to the ID to match against.
 *
 *  \param  pins    A bitmask of the buses to perform the MATCH ROM command on.
 */
void OWI_MatchRom(unsigned char * romValue, unsigned char pins)
{
	unsigned char bytesLeft = 8;   
	
	// Send the MATCH ROM command.
	OWI_SendByte(OWI_ROM_MATCH, pins);
	
	// Do once for each byte.
	while (bytesLeft > 0) {
		// Transmit 1 byte of the ID to match.
		OWI_SendByte(*romValue++, pins);
		bytesLeft--;
	}
}


/*! \brief  Sends the SEARCH ROM command and returns 1 id found on the 
 *          1-Wire(R) bus.
 *
 *  \param  bitPattern      A pointer to an 8 byte char array where the 
 *                          discovered identifier will be placed. When 
 *                          searching for several slaves, a copy of the 
 *                          last found identifier should be supplied in 
 *                          the array, or the search will fail.
 *
 *  \param  lastDeviation   The bit position where the algorithm made a 
 *                          choice the last time it was run. This argument 
 *                          should be 0 when a search is initiated. Supplying 
 *                          the return argument of this function when calling 
 *                          repeatedly will go through the complete slave 
 *                          search.
 *
 *  \param  pin             A bit-mask of the bus to perform a ROM search on.
 *
 *  \return The last bit position where there was a discrepancy between slave
 *  addresses the last time this function was run. Returns OWI_ROM_SEARCH_FAILED
 *  if an error was detected (e.g. a device was connected to the bus during the
 *  search), or OWI_ROM_SEARCH_FINISHED when there are no more devices to be
 *  discovered.
 *
 *  \note   See main.c for an example of how to utilize this function.
 */
unsigned char OWI_SearchRom(unsigned char * bitPattern,
				                unsigned char lastDeviation, unsigned char pin)
{
	unsigned char currentBit = 1;
	unsigned char newDeviation = 0;
	unsigned char bitMask = 0x01;
	unsigned char bitA;
	unsigned char bitB;
	
	// Send SEARCH ROM command on the bus.
	OWI_SendByte(OWI_ROM_SEARCH, pin);
	
	// Walk through all 64 bits.
	while (currentBit <= 64) {
		// Read bit from bus twice.
		bitA = OWI_ReadBit(pin);
		bitB = OWI_ReadBit(pin);
		
		if (bitA && bitB) {
			// Both bits 1 (Error).
			newDeviation = OWI_ROM_SEARCH_FAILED;
			return newDeviation;
		} else if (bitA ^ bitB) {
			// Bits A and B are different. All devices have the same bit here.
			// Set the bit in bitPattern to this value.
			if (bitA) {
				(*bitPattern) |= bitMask;
			} else {
				(*bitPattern) &= ~bitMask;
			}
		} else {
			// If this is where a choice was made the last time,
			// a '1' bit is selected this time.
			if (currentBit == lastDeviation) {
				(*bitPattern) |= bitMask;
			}
			
			// For the rest of the id, '0' bits are selected when
			// discrepancies occur.
			else if (currentBit > lastDeviation) {
				(*bitPattern) &= ~bitMask;
				newDeviation = currentBit;
			}
			
			// If current bit in bit pattern = 0, then this is
			// out new deviation.
			else if ( !(*bitPattern & bitMask)) {
				newDeviation = currentBit;
			}
			
		// IF the bit is already 1, do nothing.
			else {
			}
		}
		
		// Send the selected bit to the bus.
		if ((*bitPattern) & bitMask) {
			OWI_WriteBit1(pin);
		} else {
			OWI_WriteBit0(pin);
		}
		
		// Increment current bit.    
		currentBit++;
		
		// Adjust bitMask and bitPattern pointer.    
		bitMask <<= 1;
		if (!bitMask) {
			bitMask = 0x01;
			bitPattern++;
		}
	}
	
	return newDeviation;
}


/* Functions for handling CRC */
/*! \brief  Compute the CRC8 value of a data set.
 *
 *  This function will compute the CRC8 or DOW-CRC of inData using seed
 *  as inital value for the CRC.
 *
 *  \param  inData  One byte of data to compute CRC from.
 *
 *  \param  seed    The starting value of the CRC.
 *
 *  \return The CRC8 of inData with seed as initial value.
 *
 *  \note   Setting seed to 0 computes the crc8 of the inData.
 *
 *  \note   Constantly passing the return value of this function 
 *          As the seed argument computes the CRC8 value of a
 *          longer string of data.
 */
unsigned char OWI_ComputeCRC8(unsigned char inData, unsigned char seed)
{
	unsigned char bitsLeft;
	unsigned char temp;
	
	for (bitsLeft = 8; bitsLeft > 0; bitsLeft--) {
		temp = ((seed ^ inData) & 0x01);
		
		if (temp == 0) {
		seed >>= 1;
		} else {
			seed ^= 0x18;
			seed >>= 1;
			seed |= 0x80;
		}
		
		inData >>= 1;
	}
	return seed;    
}


/*! \brief  Compute the CRC16 value of a data set.
 *
 *  This function will compute the CRC16 of inData using seed
 *  as inital value for the CRC.
 *
 *  \param  inData  One byte of data to compute CRC from.
 *
 *  \param  seed    The starting value of the CRC.
 *
 *  \return The CRC16 of inData with seed as initial value.
 *
 *  \note   Setting seed to 0 computes the crc16 of the inData.
 *
 *  \note   Constantly passing the return value of this function 
 *          As the seed argument computes the CRC16 value of a
 *          longer string of data.
 */
unsigned int OWI_ComputeCRC16(unsigned char inData, unsigned int seed)
{
	unsigned char bitsLeft;
	unsigned char temp;
	
	for (bitsLeft = 8; bitsLeft > 0; bitsLeft--) {
		temp = ((seed ^ inData) & 0x01);
		
		if (temp == 0) {
			seed >>= 1;
	  } else {
			seed ^= 0x4002;
			seed >>= 1;
			seed |= 0x8000;
		}

		inData >>= 1;
	}
	
	return seed;    
}


/*! \brief  Calculate and check the CRC of a 64 bit ROM identifier.
 *  
 *  This function computes the CRC8 value of the first 56 bits of a
 *  64 bit identifier. It then checks the calculated value against the
 *  CRC value stored in ROM.
 *
 *  \param  *romValue    A pointer to an array holding a 64 bit identifier.
 *
 *  \retval OWI_CRC_OK      The CRC's matched.
 *  \retval OWI_CRC_ERROR   Calculated and stored CRC did not match.
 */
unsigned char OWI_CheckRomCRC(unsigned char *romValue)
{
	unsigned char i;
	unsigned char crc8 = 0;
	
	for (i = 0; i < 7; i++) {
		crc8 = OWI_ComputeCRC8(*romValue, crc8);
		romValue++;
	}
	
	if (crc8 == (*romValue)) {
		return OWI_CRC_OK;
	}
	
	return OWI_CRC_ERROR;
}
